# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: query.proto
# Protobuf Python Version: 6.32.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    32,
    1,
    '',
    'query.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0bquery.proto\x12\x0eimpresso.query\"%\n\tDateRange\x12\x0c\n\x04\x66rom\x18\x01 \x01(\x12\x12\n\n\x02to\x18\x02 \x01(\x12\"\x8d\x02\n\x06\x46ilter\x12.\n\x07\x63ontext\x18\x01 \x01(\x0e\x32\x1d.impresso.query.FilterContext\x12*\n\x02op\x18\x02 \x01(\x0e\x32\x1e.impresso.query.FilterOperator\x12(\n\x04type\x18\x03 \x01(\x0e\x32\x1a.impresso.query.FilterType\x12\x32\n\tprecision\x18\x04 \x01(\x0e\x32\x1f.impresso.query.FilterPrecision\x12\t\n\x01q\x18\x05 \x03(\t\x12\x30\n\tdaterange\x18\x06 \x01(\x0b\x32\x19.impresso.query.DateRangeB\x02\x18\x01\x12\x0c\n\x04uids\x18\x07 \x03(\t\"d\n\x0bSearchQuery\x12\'\n\x07\x66ilters\x18\x01 \x03(\x0b\x32\x16.impresso.query.Filter\x12,\n\x08group_by\x18\x02 \x01(\x0e\x32\x1a.impresso.query.GroupValue\"\x86\x03\n\x1e\x43ollectionRecommenderParameter\x12R\n\x03key\x18\x01 \x01(\x0e\x32\x45.impresso.query.CollectionRecommenderParameter.RecommenderParameterId\x12\x14\n\x0cstring_value\x18\x02 \x01(\t\x12\x14\n\x0cnumber_value\x18\x03 \x01(\x11\x12\x12\n\nbool_value\x18\x04 \x01(\x08\"\xcf\x01\n\x16RecommenderParameterId\x12\x12\n\x0eID_UNSPECIFIED\x10\x00\x12\x11\n\rID_COUNT_TYPE\x10\x01\x12\x16\n\x12ID_MIN_OCCURRENCES\x10\x02\x12\x15\n\x11ID_NUMBER_TO_KEEP\x10\x03\x12\x1d\n\x19ID_REMOVE_FULLY_MENTIONED\x10\x04\x12\x1a\n\x16ID_NORMALIZE_MAX_SCORE\x10\x05\x12\r\n\tID_MARGIN\x10\x06\x12\x15\n\x11ID_SCALING_FACTOR\x10\x07\"\xc1\x02\n\x15\x43ollectionRecommender\x12\x43\n\x04type\x18\x01 \x01(\x0e\x32\x35.impresso.query.CollectionRecommender.RecommenderType\x12\x0e\n\x06weight\x18\x02 \x01(\x11\x12\x42\n\nparameters\x18\x03 \x03(\x0b\x32..impresso.query.CollectionRecommenderParameter\x12\x0f\n\x07\x65nabled\x18\x04 \x01(\x08\"~\n\x0fRecommenderType\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\x13\n\x0fTYPE_TIME_RANGE\x10\x01\x12\x11\n\rTYPE_ENTITIES\x10\x02\x12\x0f\n\x0bTYPE_TOPICS\x10\x03\x12\x1c\n\x18TYPE_TEXT_REUSE_CLUSTERS\x10\x04\"]\n\x1e\x43ollectionRecommendersSettings\x12;\n\x0crecommenders\x18\x01 \x03(\x0b\x32%.impresso.query.CollectionRecommender*R\n\rFilterContext\x12\x17\n\x13\x43ONTEXT_UNSPECIFIED\x10\x00\x12\x13\n\x0f\x43ONTEXT_INCLUDE\x10\x01\x12\x13\n\x0f\x43ONTEXT_EXCLUDE\x10\x02*M\n\x0e\x46ilterOperator\x12\x18\n\x14OPERATOR_UNSPECIFIED\x10\x00\x12\x10\n\x0cOPERATOR_AND\x10\x01\x12\x0f\n\x0bOPERATOR_OR\x10\x02*\xbc\x07\n\nFilterType\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\x0c\n\x08TYPE_UID\x10\x01\x12\x1a\n\x16TYPE_HAS_TEXT_CONTENTS\x10\x02\x12\x0e\n\nTYPE_TITLE\x10\x03\x12\x11\n\rTYPE_IS_FRONT\x10\x04\x12\r\n\tTYPE_PAGE\x10\x05\x12\x0e\n\nTYPE_ISSUE\x10\x06\x12\x0f\n\x0bTYPE_STRING\x10\x07\x12\x0f\n\x0bTYPE_ENTITY\x10\x08\x12\x12\n\x0eTYPE_NEWSPAPER\x10\t\x12\x12\n\x0eTYPE_DATERANGE\x10\n\x12\r\n\tTYPE_YEAR\x10\x0b\x12\x11\n\rTYPE_LANGUAGE\x10\x0c\x12\r\n\tTYPE_TYPE\x10\r\x12\x0e\n\nTYPE_REGEX\x10\x0e\x12\x10\n\x0cTYPE_MENTION\x10\x0f\x12\x0f\n\x0bTYPE_PERSON\x10\x10\x12\x11\n\rTYPE_LOCATION\x10\x11\x12\x0e\n\nTYPE_TOPIC\x10\x12\x12\x13\n\x0fTYPE_COLLECTION\x10\x13\x12\x14\n\x10TYPE_OCR_QUALITY\x10\x14\x12\x17\n\x13TYPE_CONTENT_LENGTH\x10\x15\x12\x10\n\x0cTYPE_COUNTRY\x10\x16\x12\x15\n\x11TYPE_ACCESS_RIGHT\x10\x17\x12\x10\n\x0cTYPE_PARTNER\x10\x18\x12\x0e\n\nTYPE_MONTH\x10\x19\x12 \n\x1cTYPE_TEXT_REUSE_CLUSTER_SIZE\x10\x1a\x12+\n\'TYPE_TEXT_REUSE_CLUSTER_LEXICAL_OVERLAP\x10\x1b\x12%\n!TYPE_TEXT_REUSE_CLUSTER_DAY_DELTA\x10\x1c\x12\x1b\n\x17TYPE_TEXT_REUSE_CLUSTER\x10\x1d\x12\x19\n\x15TYPE_MENTION_FUNCTION\x10\x1e\x12\x0c\n\x08TYPE_NAG\x10\x1f\x12\x14\n\x10TYPE_WIKIDATA_ID\x10 \x12\x14\n\x10TYPE_DATA_DOMAIN\x10!\x12\x12\n\x0eTYPE_COPYRIGHT\x10\"\x12\x14\n\x10TYPE_SOURCE_TYPE\x10#\x12\x16\n\x12TYPE_SOURCE_MEDIUM\x10$\x12\x15\n\x11TYPE_ORGANISATION\x10%\x12\x12\n\x0eTYPE_EMBEDDING\x10&\x12\x1d\n\x19TYPE_IMAGE_VISUAL_CONTENT\x10\'\x12\x18\n\x14TYPE_IMAGE_TECHNIQUE\x10(\x12!\n\x1dTYPE_IMAGE_COMMUNICATION_GOAL\x10)\x12\x1b\n\x17TYPE_IMAGE_CONTENT_TYPE\x10**\x81\x01\n\x0f\x46ilterPrecision\x12\x19\n\x15PRECISION_UNSPECIFIED\x10\x00\x12\x13\n\x0fPRECISION_EXACT\x10\x01\x12\x15\n\x11PRECISION_PARTIAL\x10\x02\x12\x13\n\x0fPRECISION_FUZZY\x10\x03\x12\x12\n\x0ePRECISION_SOFT\x10\x04*A\n\nGroupValue\x12\x1a\n\x16GROUPVALUE_UNSPECIFIED\x10\x00\x12\x17\n\x13GROUPVALUE_ARTICLES\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'query_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_FILTER'].fields_by_name['daterange']._loaded_options = None
  _globals['_FILTER'].fields_by_name['daterange']._serialized_options = b'\030\001'
  _globals['_FILTERCONTEXT']._serialized_start=1256
  _globals['_FILTERCONTEXT']._serialized_end=1338
  _globals['_FILTEROPERATOR']._serialized_start=1340
  _globals['_FILTEROPERATOR']._serialized_end=1417
  _globals['_FILTERTYPE']._serialized_start=1420
  _globals['_FILTERTYPE']._serialized_end=2376
  _globals['_FILTERPRECISION']._serialized_start=2379
  _globals['_FILTERPRECISION']._serialized_end=2508
  _globals['_GROUPVALUE']._serialized_start=2510
  _globals['_GROUPVALUE']._serialized_end=2575
  _globals['_DATERANGE']._serialized_start=31
  _globals['_DATERANGE']._serialized_end=68
  _globals['_FILTER']._serialized_start=71
  _globals['_FILTER']._serialized_end=340
  _globals['_SEARCHQUERY']._serialized_start=342
  _globals['_SEARCHQUERY']._serialized_end=442
  _globals['_COLLECTIONRECOMMENDERPARAMETER']._serialized_start=445
  _globals['_COLLECTIONRECOMMENDERPARAMETER']._serialized_end=835
  _globals['_COLLECTIONRECOMMENDERPARAMETER_RECOMMENDERPARAMETERID']._serialized_start=628
  _globals['_COLLECTIONRECOMMENDERPARAMETER_RECOMMENDERPARAMETERID']._serialized_end=835
  _globals['_COLLECTIONRECOMMENDER']._serialized_start=838
  _globals['_COLLECTIONRECOMMENDER']._serialized_end=1159
  _globals['_COLLECTIONRECOMMENDER_RECOMMENDERTYPE']._serialized_start=1033
  _globals['_COLLECTIONRECOMMENDER_RECOMMENDERTYPE']._serialized_end=1159
  _globals['_COLLECTIONRECOMMENDERSSETTINGS']._serialized_start=1161
  _globals['_COLLECTIONRECOMMENDERSSETTINGS']._serialized_end=1254
# @@protoc_insertion_point(module_scope)
